/************************************************************************
@@Smoke.cpp

@@Smoke Effect

@@Date : Nov. 12, 2007
@@Version : 1.0
@@Author : Pocol
************************************************************************/

//
//@include
//
#include "Smoke.h"

//
//@global
//
double Random() { return (double)rand()/(double)RAND_MAX; }
double Random05() { return Random()-0.5; }
const static double InitXZ = 0.0;
const static double MaxXZ = 2.0;
const static double InitY = 0.0;
const static double MinSize = 0.3;
const static double MaxSize = 0.5;
const static double MinVY = 0.01;
const static double MaxVY = 0.02;
const static double RandColor= 0.2;
const static int MinTime = 20;
const static int MaxTime = 100;

/////////////////////////////////////////////////////////////////////////
//@@SmokeParticle
/////////////////////////////////////////////////////////////////////////

//---------------------------------------------------------------------------------------------------
//@@SmokeParticle
//@@Desc : RXgN^
//---------------------------------------------------------------------------------------------------
SmokeParticle::SmokeParticle(float base_color[])
{
	for ( int i=0; i<3; i++ )
		baseColor[i] = base_color[i];

	Initialize();

	for ( int i=0, n=Random()*MaxTime; i<n; i++ ) Move();
}

//----------------------------------------------------------------------------------------------------
//@@Initialize
//@@Desc : 
//----------------------------------------------------------------------------------------------------
void SmokeParticle::Initialize()
{
	double rad = Random()*3.14*2.0;
	double xz= Random05()*MaxXZ;
	
	position[0] = InitXZ + cos(rad)*xz;
	position[1] = InitY;
	position[2] = Random05()*MaxXZ;

	velocity[0] = 0.0;
	velocity[1] = MinVY + (MaxVY - MinVY)*Random();
	velocity[2] = 0.0;

	size = MinSize + (MaxSize - MinSize)*Random();

	float c= Random05()*RandColor;
	color[0] = baseColor[0]+ c;
	color[1] = baseColor[1] + c;
	color[2] = baseColor[2] + c;

	time = 0;
	endTime = MinTime + (MaxTime - MinTime)*Random();
}

//----------------------------------------------------------------------------------------------------
//@@Move
//@@Desc : ړ
//----------------------------------------------------------------------------------------------------
void SmokeParticle::Move()
{
	for ( int i=0; i<3; i++ )
	{
		position[i] += velocity[i];
	}
	time++;

	if ( time >= endTime ) Initialize();
}

//----------------------------------------------------------------------------------------------------
//@@Render
//@@Desc : `揈
//----------------------------------------------------------------------------------------------------
void SmokeParticle::Render()
{
	GLfloat mat_color[4] = { color[0], color[1], color[2], sin(3.14*(float)(endTime - time)/endTime) };
	glColor4fv(mat_color);
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, mat_color);
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mat_color);
	glTexEnvfv(GL_TEXTURE_ENV, GL_TEXTURE_ENV_COLOR, mat_color);

	glBegin(GL_QUADS);
	glTexCoord2f(0.0, 0.0);	glVertex3d(position[0] - size, position[1] - size, position[2]);
	glTexCoord2f(0.0, 1.0);	glVertex3d(position[0] - size, position[1] + size, position[2]);
	glTexCoord2f(1.0, 1.0);		glVertex3d(position[0] + size, position[1] + size, position[2]);
	glTexCoord2f(1.0, 0.0);	glVertex3d(position[0] + size, position[1] - size, position[2]);
	glEnd();
}

/////////////////////////////////////////////////////////////////////////
//@@Smoke class
/////////////////////////////////////////////////////////////////////////

//-----------------------------------------------------------------------------------------------------
//@@Smoke
//@@Desc : RXgN^
//-----------------------------------------------------------------------------------------------------
Smoke::Smoke(int smoke_count, float smoke_color[]) : smokeCount(smoke_count)
{
	smokeparticle = new SmokeParticle *[smokeCount];
	for ( int i=0; i<smokeCount; i++ )
	{
		smokeparticle[i] = new SmokeParticle(smoke_color);
	}
}

//-----------------------------------------------------------------------------------------------------
//@@~Smoke
//@@Desc : fXgN^
//-----------------------------------------------------------------------------------------------------
Smoke::~Smoke()
{
	delete [] smokeparticle;
}

//----------------------------------------------------------------------------------------------------
//@@LoadTexture
//@@Desc : eNX`̃[h
//----------------------------------------------------------------------------------------------------
void Smoke::LoadTexture(const char *filename)
{
	texture.Load(filename);

	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_BLEND);
}

//-----------------------------------------------------------------------------------------------------
//@@Move
//@@Desc : ړ
//-----------------------------------------------------------------------------------------------------
void Smoke::Move()
{
	for ( int i=0; i<smokeCount; i++ )
	{
		smokeparticle[i]->Move();
	}
}

//---------------------------------------------------------------------------------------------------
//@@Render
//@@Desc : `揈
//---------------------------------------------------------------------------------------------------
void Smoke::Render()
{
	glDisable(GL_DEPTH_TEST);

	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_BLEND);

	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, texture.ID);

	for ( int i=0; i<smokeCount; i++ )
	{
		smokeparticle[i]->Render();
	}

	glDisable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, 0);

	glDisable(GL_BLEND);

	glEnable(GL_DEPTH_TEST);
}